@@ -23,7 +23,6 @@
 
 #include <cmsys/SystemTools.hxx>
 #include <cmsys/Glob.hxx>
-#include <memory> // auto_ptr
 #include <algorithm>
 
 #if defined(__HAIKU__)
@@ -62,10 +61,31 @@ void cmCPackGenerator::DisplayVerboseOutput(const char* msg,
 
 //----------------------------------------------------------------------
 int cmCPackGenerator::PrepareNames()
-{  
+{
   cmCPackLogger(cmCPackLog::LOG_DEBUG,
     "Create temp directory." << std::endl);
 
+  // checks CPACK_SET_DESTDIR support
+  if (IsOn("CPACK_SET_DESTDIR"))
+    {
+      if (SETDESTDIR_UNSUPPORTED==SupportsSetDestdir())
+        {
+          cmCPackLogger(cmCPackLog::LOG_ERROR,
+                        "CPACK_SET_DESTDIR is set to ON but the '"
+                        << Name << "' generator does NOT support it."
+                        << std::endl);
+          return 0;
+        }
+      else if (SETDESTDIR_SHOULD_NOT_BE_USED==SupportsSetDestdir())
+        {
+          cmCPackLogger(cmCPackLog::LOG_WARNING,
+                        "CPACK_SET_DESTDIR is set to ON but it is "
+                        << "usually a bad idea to do that with '"
+                        << Name << "' generator. Use at your own risk."
+                        << std::endl);
+        }
+  }
+
   std::string tempDirectory = this->GetOption("CPACK_PACKAGE_DIRECTORY");
   tempDirectory += "/_CPack_Packages/";
   const char* toplevelTag = this->GetOption("CPACK_TOPLEVEL_TAG");
@@ -76,7 +96,6 @@ int cmCPackGenerator::PrepareNames()
     }
   tempDirectory += this->GetOption("CPACK_GENERATOR");
   std::string topDirectory = tempDirectory;
-  this->GetOption("CPACK_PACKAGE_FILE_NAME");
   const char* pfname = this->GetOption("CPACK_PACKAGE_FILE_NAME");
   if(!pfname)
     {
@@ -84,7 +103,7 @@ int cmCPackGenerator::PrepareNames()
                   "CPACK_PACKAGE_FILE_NAME not specified" << std::endl);
     return 0;
     }
-  std::string outName = pfname; 
+  std::string outName = pfname;
   tempDirectory += "/" + outName;
   if(!this->GetOutputExtension())
     {
@@ -128,7 +147,7 @@ int cmCPackGenerator::PrepareNames()
     if ( !cmSystemTools::FileExists(descFileName) )
       {
       cmCPackLogger(cmCPackLog::LOG_ERROR,
-                    "Cannot find description file name: [" 
+                    "Cannot find description file name: ["
                     << descFileName << "]" << std::endl);
       return 0;
       }
@@ -186,8 +205,8 @@ int cmCPackGenerator::InstallProject()
   if ( !cmsys::SystemTools::MakeDirectory(bareTempInstallDirectory.c_str()))
     {
     cmCPackLogger(cmCPackLog::LOG_ERROR,
-      "Problem creating temporary directory: " 
-                  << (tempInstallDirectory ? tempInstallDirectory : "(NULL}")
+      "Problem creating temporary directory: "
+                  << (tempInstallDirectory ? tempInstallDirectory : "(NULL)")
                   << std::endl);
     return 0;
     }
@@ -388,8 +407,11 @@ int cmCPackGenerator::InstallProjectViaInstalledDirectories(
                                    std::string>(targetFile,inFileRelative));
           }
         /* If it is not a symlink then do a plain copy */
-        else if ( !cmSystemTools::CopyFileIfDifferent(inFile.c_str(),
-            filePath.c_str()) )
+        else if (!(
+            cmSystemTools::CopyFileIfDifferent(inFile.c_str(),filePath.c_str())
+            &&
+            cmSystemTools::CopyFileTime(inFile.c_str(),filePath.c_str())
+                ) )
           {
           cmCPackLogger(cmCPackLog::LOG_ERROR, "Problem copying file: "
             << inFile.c_str() << " -> " << filePath.c_str() << std::endl);
@@ -560,7 +582,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
           !(this->IsSet("CPACK_MONOLITHIC_INSTALL")))
         {
         // Determine the installation types for this project (if provided).
-        std::string installTypesVar = "CPACK_" 
+        std::string installTypesVar = "CPACK_"
           + cmSystemTools::UpperCase(installComponent) + "_INSTALL_TYPES";
         const char *installTypes = this->GetOption(installTypesVar.c_str());
         if (installTypes && *installTypes)
@@ -572,16 +594,16 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
                installTypeIt != installTypesVector.end();
                ++installTypeIt)
             {
-            this->GetInstallationType(installProjectName.c_str(), 
+            this->GetInstallationType(installProjectName.c_str(),
                                       installTypeIt->c_str());
             }
           }
 
         // Determine the set of components that will be used in this project
-        std::string componentsVar 
+        std::string componentsVar
           = "CPACK_COMPONENTS_" + cmSystemTools::UpperCase(installComponent);
         const char *components = this->GetOption(componentsVar.c_str());
-        if (components && *components) 
+        if (components && *components)
           {
           cmSystemTools::ExpandListArgument(components, componentsVector);
           std::vector<std::string>::iterator compIt;
@@ -616,7 +638,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
           = this->MakefileMap->GetDefinition("CMAKE_MAKE_PROGRAM");
         std::string buildCommand
           = globalGenerator->GenerateBuildCommand(cmakeMakeProgram,
-            installProjectName.c_str(), 0,
+            installProjectName.c_str(), 0, 0,
             globalGenerator->GetPreinstallTargetName(),
             buildConfig, false, false);
         cmCPackLogger(cmCPackLog::LOG_DEBUG,
@@ -664,16 +686,16 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
         if (componentInstall)
           {
           cmCPackLogger(cmCPackLog::LOG_OUTPUT,
-                        "-   Install component: " << installComponent 
+                        "-   Install component: " << installComponent
                         << std::endl);
           }
-        
+
         cmake cm;
         cm.AddCMakePaths();
         cm.SetProgressCallback(cmCPackGeneratorProgress, this);
         cmGlobalGenerator gg;
         gg.SetCMakeInstance(&cm);
-        std::auto_ptr<cmLocalGenerator> lg(gg.CreateLocalGenerator());
+        cmsys::auto_ptr<cmLocalGenerator> lg(gg.CreateLocalGenerator());
         cmMakefile *mf = lg->GetMakefile();
         std::string realInstallDirectory = tempInstallDirectory;
         if ( !installSubDirectory.empty() && installSubDirectory != "/" )
@@ -734,7 +756,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
             "- Using DESTDIR + CPACK_INSTALL_PREFIX... (mf->AddDefinition)"
             << std::endl);
           cmCPackLogger(cmCPackLog::LOG_DEBUG,
-                        "- Setting CMAKE_INSTALL_PREFIX to '" << dir << "'" 
+                        "- Setting CMAKE_INSTALL_PREFIX to '" << dir << "'"
                         << std::endl);
 
           // Make sure that DESTDIR + CPACK_INSTALL_PREFIX directory
@@ -768,7 +790,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
           if ( !cmsys::SystemTools::MakeDirectory(dir.c_str()))
             {
             cmCPackLogger(cmCPackLog::LOG_ERROR,
-                          "Problem creating temporary directory: " 
+                          "Problem creating temporary directory: "
                           << dir << std::endl);
             return 0;
             }
@@ -782,7 +804,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
                  tempInstallDirectory.c_str()))
             {
             cmCPackLogger(cmCPackLog::LOG_ERROR,
-                          "Problem creating temporary directory: " 
+                          "Problem creating temporary directory: "
                           << tempInstallDirectory << std::endl);
             return 0;
             }
@@ -808,7 +830,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
                             installComponent.c_str());
           }
 
-        // strip on TRUE, ON, 1, one or several file names, but not on 
+        // strip on TRUE, ON, 1, one or several file names, but not on
         // FALSE, OFF, 0 and an empty string
         if (!cmSystemTools::IsOff(this->GetOption("CPACK_STRIP_FILES")))
           {
@@ -828,8 +850,35 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
           filesBefore = glB.GetFiles();
           std::sort(filesBefore.begin(),filesBefore.end());
           }
+
+        // If CPack was asked to warn on ABSOLUTE INSTALL DESTINATION
+        // then forward request to cmake_install.cmake script
+        if (this->IsOn("CPACK_WARN_ON_ABSOLUTE_INSTALL_DESTINATION"))
+          {
+            mf->AddDefinition("CMAKE_WARN_ON_ABSOLUTE_INSTALL_DESTINATION",
+                              "1");
+          }
+        // If current CPack generator does support
+        // ABSOLUTE INSTALL DESTINATION or CPack has been asked for
+        // then ask cmake_install.cmake script to error out
+        // as soon as it occurs (before installing file)
+        if (!SupportsAbsoluteDestination() ||
+            this->IsOn("CPACK_ERROR_ON_ABSOLUTE_INSTALL_DESTINATION"))
+          {
+            mf->AddDefinition("CMAKE_ERROR_ON_ABSOLUTE_INSTALL_DESTINATION",
+                              "1");
+          }
         // do installation
         int res = mf->ReadListFile(0, installFile.c_str());
+        // forward definition of CMAKE_ABSOLUTE_DESTINATION_FILES
+        // to CPack (may be used by generators like CPack RPM or DEB)
+        // in order to transparently handle ABSOLUTE PATH
+        if (mf->GetDefinition("CMAKE_ABSOLUTE_DESTINATION_FILES"))
+          {
+            mf->AddDefinition("CPACK_ABSOLUTE_DESTINATION_FILES",
+                mf->GetDefinition("CMAKE_ABSOLUTE_DESTINATION_FILES"));
+          }
+
         // Now rebuild the list of files after installation
         // of the current component (if we are in component install)
         if (componentInstall)
@@ -854,7 +903,7 @@ int cmCPackGenerator::InstallProjectViaInstallCMakeProjects(
             localFileName =
                 cmSystemTools::RelativePath(InstallPrefix, fit->c_str());
             localFileName =
-                localFileName.substr(localFileName.find('/')+1,
+                localFileName.substr(localFileName.find_first_not_of('/'),
                                      std::string::npos);
             Components[installComponent].Files.push_back(localFileName);
             cmCPackLogger(cmCPackLog::LOG_DEBUG, "Adding file <"
@@ -953,6 +1002,8 @@ int cmCPackGenerator::DoPackage()
   cmCPackLogger(cmCPackLog::LOG_OUTPUT,
     "Create package using " << this->Name.c_str() << std::endl);
 
+  // Prepare CPack internal name and check
+  // values for many CPACK_xxx vars
   if ( !this->PrepareNames() )
     {
     return 0;
@@ -971,7 +1022,7 @@ int cmCPackGenerator::DoPackage()
       = this->GetOption("CPACK_TOPLEVEL_DIRECTORY");
     if ( cmSystemTools::FileExists(toplevelDirectory) )
       {
-      cmCPackLogger(cmCPackLog::LOG_VERBOSE, 
+      cmCPackLogger(cmCPackLog::LOG_VERBOSE,
                     "Remove toplevel directory: "
         << toplevelDirectory << std::endl);
       if ( !cmSystemTools::RepeatedRemoveDirectory(toplevelDirectory) )
@@ -1100,10 +1151,10 @@ int cmCPackGenerator::Initialize(const char* name, cmMakefile* mf)
   // set the running generator name
   this->SetOption("CPACK_GENERATOR", this->Name.c_str());
   // Load the project specific config file
-  const char* config = 
+  const char* config =
     this->GetOption("CPACK_PROJECT_CONFIG_FILE");
   if(config)
-    { 
+    {
     mf->ReadListFile(config);
     }
   int result = this->InitializeInternal();
@@ -1140,13 +1191,13 @@ bool cmCPackGenerator::IsOn(const char* name) const
 
 //----------------------------------------------------------------------
 const char* cmCPackGenerator::GetOption(const char* op) const
-{ 
+{
   const char* ret = this->MakefileMap->GetDefinition(op);
   if(!ret)
-    { 
-    cmCPackLogger(cmCPackLog::LOG_DEBUG, 
+    {
+    cmCPackLogger(cmCPackLog::LOG_DEBUG,
                   "Warning, GetOption return NULL for: "
-                  << op 
+                  << op
                   << std::endl);
     }
   return ret;
@@ -1169,7 +1220,7 @@ int cmCPackGenerator::SetCMakeRoot()
       return 1;
     }
   cmCPackLogger(cmCPackLog::LOG_ERROR,
-                "Could not find CMAKE_ROOT !!!" 
+                "Could not find CMAKE_ROOT !!!"
                 << std::endl
                 << "CMake has most likely not been installed correctly."
                 << std::endl
@@ -1430,6 +1481,19 @@ std::string cmCPackGenerator::GetComponentPackageFileName(
   return initialPackageFileName + suffix;
 }
 
+//----------------------------------------------------------------------
+enum cmCPackGenerator::CPackSetDestdirSupport
+cmCPackGenerator::SupportsSetDestdir() const
+{
+  return cmCPackGenerator::SETDESTDIR_SUPPORTED;
+}
+
+//----------------------------------------------------------------------
+bool cmCPackGenerator::SupportsAbsoluteDestination() const
+{
+  return true;
+}
+
 //----------------------------------------------------------------------
 bool cmCPackGenerator::SupportsComponentInstallation() const
 {
@@ -1450,14 +1514,14 @@ cmCPackGenerator::GetInstallationType(const char *projectName,
   (void) projectName;
   bool hasInstallationType = this->InstallationTypes.count(name) != 0;
   cmCPackInstallationType *installType = &this->InstallationTypes[name];
-  if (!hasInstallationType) 
+  if (!hasInstallationType)
     {
     // Define the installation type
-    std::string macroPrefix = "CPACK_INSTALL_TYPE_" 
+    std::string macroPrefix = "CPACK_INSTALL_TYPE_"
       + cmsys::SystemTools::UpperCase(name);
     installType->Name = name;
 
-    const char* displayName 
+    const char* displayName
       = this->GetOption((macroPrefix + "_DISPLAY_NAME").c_str());
     if (displayName && *displayName)
       {
@@ -1480,13 +1544,13 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
 {
   bool hasComponent = this->Components.count(name) != 0;
   cmCPackComponent *component = &this->Components[name];
-  if (!hasComponent) 
+  if (!hasComponent)
     {
     // Define the component
-    std::string macroPrefix = "CPACK_COMPONENT_" 
+    std::string macroPrefix = "CPACK_COMPONENT_"
       + cmsys::SystemTools::UpperCase(name);
     component->Name = name;
-    const char* displayName 
+    const char* displayName
       = this->GetOption((macroPrefix + "_DISPLAY_NAME").c_str());
     if (displayName && *displayName)
       {
@@ -1496,7 +1560,7 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
       {
       component->DisplayName = component->Name;
       }
-    component->IsHidden 
+    component->IsHidden
       = this->IsSet((macroPrefix + "_HIDDEN").c_str());
     component->IsRequired
       = this->IsSet((macroPrefix + "_REQUIRED").c_str());
@@ -1506,7 +1570,7 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
       = this->IsSet((macroPrefix + "_DOWNLOADED").c_str())
         || cmSystemTools::IsOn(this->GetOption("CPACK_DOWNLOAD_ALL"));
 
-    const char* archiveFile = this->GetOption((macroPrefix + 
+    const char* archiveFile = this->GetOption((macroPrefix +
                                                "_ARCHIVE_FILE").c_str());
     if (archiveFile && *archiveFile)
       {
@@ -1514,7 +1578,7 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
       }
 
     const char* groupName = this->GetOption((macroPrefix + "_GROUP").c_str());
-    if (groupName && *groupName) 
+    if (groupName && *groupName)
       {
       component->Group = GetComponentGroup(projectName, groupName);
       component->Group->Components.push_back(component);
@@ -1532,15 +1596,15 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
       }
 
     // Determine the installation types.
-    const char *installTypes 
+    const char *installTypes
       = this->GetOption((macroPrefix + "_INSTALL_TYPES").c_str());
-    if (installTypes && *installTypes) 
+    if (installTypes && *installTypes)
       {
       std::vector<std::string> installTypesVector;
       cmSystemTools::ExpandListArgument(installTypes, installTypesVector);
       std::vector<std::string>::iterator installTypesIt;
-      for (installTypesIt = installTypesVector.begin(); 
-           installTypesIt != installTypesVector.end(); 
+      for (installTypesIt = installTypesVector.begin();
+           installTypesIt != installTypesVector.end();
            ++installTypesIt)
         {
         component->InstallationTypes.push_back(
@@ -1550,16 +1614,16 @@ cmCPackGenerator::GetComponent(const char *projectName, const char *name)
 
     // Determine the component dependencies.
     const char *depends = this->GetOption((macroPrefix + "_DEPENDS").c_str());
-    if (depends && *depends) 
+    if (depends && *depends)
       {
       std::vector<std::string> dependsVector;
       cmSystemTools::ExpandListArgument(depends, dependsVector);
       std::vector<std::string>::iterator dependIt;
-      for (dependIt = dependsVector.begin(); 
-           dependIt != dependsVector.end(); 
+      for (dependIt = dependsVector.begin();
+           dependIt != dependsVector.end();
            ++dependIt)
         {
-        cmCPackComponent *child = GetComponent(projectName, 
+        cmCPackComponent *child = GetComponent(projectName,
                                                dependIt->c_str());
         component->Dependencies.push_back(child);
         child->ReverseDependencies.push_back(component);
@@ -1574,15 +1638,15 @@ cmCPackComponentGroup*
 cmCPackGenerator::GetComponentGroup(const char *projectName, const char *name)
 {
   (void) projectName;
-  std::string macroPrefix = "CPACK_COMPONENT_GROUP_" 
+  std::string macroPrefix = "CPACK_COMPONENT_GROUP_"
         + cmsys::SystemTools::UpperCase(name);
   bool hasGroup = this->ComponentGroups.count(name) != 0;
   cmCPackComponentGroup *group = &this->ComponentGroups[name];
-  if (!hasGroup) 
+  if (!hasGroup)
     {
     // Define the group
     group->Name = name;
-    const char* displayName 
+    const char* displayName
       = this->GetOption((macroPrefix + "_DISPLAY_NAME").c_str());
     if (displayName && *displayName)
       {
@@ -1603,7 +1667,7 @@ cmCPackGenerator::GetComponentGroup(const char *projectName, const char *name)
       = this->IsSet((macroPrefix + "_BOLD_TITLE").c_str());
     group->IsExpandedByDefault
       = this->IsSet((macroPrefix + "_EXPANDED").c_str());
-    const char* parentGroupName 
+    const char* parentGroupName
       = this->GetOption((macroPrefix + "_PARENT_GROUP").c_str());
     if (parentGroupName && *parentGroupName)
       {
