@@ -26,7 +26,7 @@ public:
   /**
    * This is a virtual constructor for the command.
    */
-  virtual cmCommand* Clone() 
+  virtual cmCommand* Clone()
     {
     return new cmAddTestCommand;
     }
@@ -41,20 +41,20 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "add_test";}
+  virtual const char* GetName() const { return "add_test";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "Add a test to the project with the specified arguments.";
     }
-  
+
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
       "  add_test(testname Exename arg1 arg2 ... )\n"
@@ -71,6 +71,9 @@ public:
       "  add_test(NAME <name> [CONFIGURATIONS [Debug|Release|...]]\n"
       "           [WORKING_DIRECTORY dir]\n"
       "           COMMAND <command> [arg1 [arg2 ...]])\n"
+      "Add a test called <name>.  "
+      "The test name may not contain spaces, quotes, or other characters "
+      "special in CMake syntax.  "
       "If COMMAND specifies an executable target (created by "
       "add_executable) it will automatically be replaced by the location "
       "of the executable created at build time.  "
@@ -81,7 +84,7 @@ public:
       "\n"
       "Arguments after COMMAND may use \"generator expressions\" with the "
       "syntax \"$<...>\".  "
-      CM_DOCUMENT_COMMAND_GENERATOR_EXPRESSIONS
+      CM_DOCUMENT_ADD_TEST_GENERATOR_EXPRESSIONS
       "Example usage:\n"
       "  add_test(NAME mytest\n"
       "           COMMAND testDriver --config $<CONFIGURATION>\n"
@@ -91,7 +94,7 @@ public:
       "executable file produced by target \"myexe\"."
       ;
     }
-  
+
   cmTypeMacro(cmAddTestCommand, cmCommand);
 private:
   bool HandleNameMode(std::vector<std::string> const& args);
