@@ -17,8 +17,8 @@
 /** \class cmCMakePolicyCommand
  * \brief Set how CMake should handle policies
  *
- * cmCMakePolicyCommand sets how CMake should deal with backwards 
- * compatibility policies.   
+ * cmCMakePolicyCommand sets how CMake should deal with backwards
+ * compatibility policies.
  */
 class cmCMakePolicyCommand : public cmCommand
 {
@@ -26,7 +26,7 @@ public:
   /**
    * This is a virtual constructor for the command.
    */
-  virtual cmCommand* Clone() 
+  virtual cmCommand* Clone()
     {
     return new cmCMakePolicyCommand;
     }
@@ -41,25 +41,25 @@ public:
  /**
    * This determines if the command is invoked when in script mode.
    */
-  virtual bool IsScriptable() { return true; }
+  virtual bool IsScriptable() const { return true; }
 
  /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() {return "cmake_policy";}
-  
+  virtual const char* GetName() const {return "cmake_policy";}
+
  /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "Manage CMake Policy settings.";
     }
-  
+
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
       "As CMake evolves it is sometimes necessary to change existing "
@@ -129,7 +129,7 @@ public:
       "closest nested policy stack entry."
       ;
     }
-  
+
   cmTypeMacro(cmCMakePolicyCommand, cmCommand);
 private:
   bool HandleSetMode(std::vector<std::string> const& args);
