@@ -61,7 +61,7 @@ bool cmIncludeCommand
       noPolicyScope = true;
       }
       else if(i > 1)  // compat.: in previous cmake versions the second
-                      // parameter was ignore if it wasn't "OPTIONAL"
+                      // parameter was ignored if it wasn't "OPTIONAL"
         {
         std::string errorText = "called with invalid argument: ";
         errorText += args[i];
@@ -70,6 +70,13 @@ bool cmIncludeCommand
         }
     }
 
+  if(fname.empty())
+    {
+    this->Makefile->IssueMessage(cmake::AUTHOR_WARNING,
+                                 "include() given empty file name (ignored).");
+    return true;
+    }
+
   if(!cmSystemTools::FileIsFullPath(fname.c_str()))
     {
     // Not a path. Maybe module.
