@@ -64,18 +64,19 @@ public:
       "Specify targetName to build a specific target instead of the 'all' or "
       "'ALL_BUILD' target."
       "\n"
-      "  try_compile(RESULT_VAR <bindir> <srcfile>\n"
+      "  try_compile(RESULT_VAR <bindir> <srcfile|SOURCES srcfile...>\n"
       "              [CMAKE_FLAGS flags...]\n"
       "              [COMPILE_DEFINITIONS flags...]\n"
+      "              [LINK_LIBRARIES libs...]\n"
       "              [OUTPUT_VARIABLE <var>]\n"
-      "              [COPY_FILE <fileName>])\n"
-      "Try building a source file into an executable.  "
-      "In this form the user need only supply a source file that defines "
-      "a 'main'.  "
-      "CMake will create a CMakeLists.txt file to build the source "
+      "              [COPY_FILE <fileName> [COPY_FILE_ERROR <var>]])\n"
+      "Try building an executable from one or more source files.  "
+      "In this form the user need only supply one or more source files "
+      "that include a definition for 'main'.  "
+      "CMake will create a CMakeLists.txt file to build the source(s) "
       "as an executable.  "
       "Specify COPY_FILE to get a copy of the linked executable at the "
-      "given fileName."
+      "given fileName and optionally COPY_FILE_ERROR to capture any error."
       "\n"
       "In this version all files in bindir/CMakeFiles/CMakeTmp "
       "will be cleaned automatically. For debugging, --debug-trycompile can "
@@ -90,7 +91,12 @@ public:
       "Some extra flags that can be included are,  "
       "INCLUDE_DIRECTORIES, LINK_DIRECTORIES, and LINK_LIBRARIES.  "
       "COMPILE_DEFINITIONS are -Ddefinition that will be passed to the "
-      "compile line.  "
+      "compile line.\n"
+      "The srcfile signature also accepts a LINK_LIBRARIES argument which "
+      "may contain a list of libraries or IMPORTED targets which will be "
+      "linked to in the generated project.  If LINK_LIBRARIES is specified "
+      "as a parameter to try_compile, then any LINK_LIBRARIES passed as "
+      "CMAKE_FLAGS will be ignored.\n"
       "try_compile creates a CMakeList.txt "
       "file on the fly that looks like this:\n"
       "  add_definitions( <expanded COMPILE_DEFINITIONS from calling "
